# WARNING: Please don't edit this file. It was generated by Python/WinRT v2.3.0

import enum

import winrt.system
from winrt import _winrt_windows_devices_adc

__all__ = [
    "AdcChannelMode",
    "AdcChannel",
    "AdcController",
]

class AdcChannelMode(enum.IntEnum):
    SINGLE_ENDED = 0
    DIFFERENTIAL = 1

AdcChannel = _winrt_windows_devices_adc.AdcChannel
AdcController = _winrt_windows_devices_adc.AdcController
